﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.Generic;
using System.IO;
using System.Xml.Linq;
using System.Linq;
using MediaInfo;

namespace Thumbnails.Data
{
  public class MediaFileLoadedEventArgs : EventArgs
  {
    public IEnumerable<MediaEx> Collection
    {
      get;
      set;
    }
    public Exception Error
    {
      get;
      set;
    }
  }

  public class MediaInformationFile
  {
    public const string MEDIA_FILE_NAME = "media.xml";
    public const string MEDIA_PATH = "MediaFiles";

    private XDocument _document;

    public event EventHandler Loaded;

    private void OnMediaFileLoaded(IEnumerable<MediaEx> xmlMedias,
      Exception lastError)
    {
      if (Loaded != null)
      {
        MediaFileLoadedEventArgs args = new MediaFileLoadedEventArgs();
        args.Collection = xmlMedias;
        args.Error = lastError;
        Loaded(this, args);
      }
    }

    public void LoadMediaFile()
    {
      WebClient client = new WebClient();
      client.DownloadStringCompleted
        += new DownloadStringCompletedEventHandler(
          client_DownloadStringCompleted);
      client.DownloadStringAsync(new Uri(MEDIA_FILE_NAME, UriKind.Relative));
    }

    void client_DownloadStringCompleted(object sender,
      DownloadStringCompletedEventArgs e)
    {
      // Prepare data
      IEnumerable<MediaEx> output = null;
      Exception lastError = null;
      try
      {
        StringReader reader = new StringReader(e.Result);
        _document = XDocument.Load(reader);
        output = from xmlMediaInfo
          in _document.Descendants("media")
          select new MediaEx
          {
            MediaPath = MEDIA_PATH,
            MediaName = (string) xmlMediaInfo.Attribute("name"),
            Type = (MediaType) Enum.Parse(typeof(MediaType),
              (string) xmlMediaInfo.Attribute("type"), true),
            Description = (string) xmlMediaInfo.Attribute("description")
          };
      }
      catch (Exception ex)
      {
        // Failure, inform user
        if (ex.InnerException != null
          && ex.InnerException is WebException)
        {
          lastError = ex.InnerException;
        }
        else
        {
          lastError = ex;
        }

        output = null;
      }

      OnMediaFileLoaded(output, lastError);
    }
  }
}
